/****************************************************************/
/* Filename:     MEMMGR.C                                       */
/*                                                              */
/* Description:  Program source                                 */
/*--------------------------------------------------------------*/

#include "windows.h"
#include "memmgr.h"

typedef BOOL FAR * LPBOOL;

  HANDLE                hInst;
  FARPROC               lpAllocProc;
  int                   i,              index;
  char                  szString[80],   szNumber[17],   szStr[4];
  HMENU                 hMenu;
  HANDLE                hOldMem;
  unsigned int          flags;

struct {
  HANDLE                hMem;           /* handle to memory block  */
  unsigned int          flags;          /* allocation flags        */
  BOOL                  colour;         /* T: Global  F: Local     */
  unsigned int          size;           /* block size              */
  } memory[16];

  unsigned int          item            = 0;    /* current item */
  BOOL                  bValOK;
  BOOL                  bLocalFreeze    = FALSE;
  BOOL                  bLockData       = FALSE;

/*==============================================================*/

long FAR PASCAL MainWndProc (hWnd, msg, wParam, lParam)
  HWND          hWnd;
  unsigned      msg;
  WORD          wParam;
  LONG          lParam;
  {
  PAINTSTRUCT   ps;
  int           lineNum;

  switch (msg)  {
    case WM_CREATE:
      hMenu = GetMenu (hWnd);
      EnableMenuItem (hMenu, IDLOCALMELT, MF_GRAYED);
      EnableMenuItem (hMenu, IDUNLOCKDATA, MF_GRAYED);
      break;

    case WM_DESTROY:
      for (i=0; i<16; i++)              /* Free any allocations */
        if (memory[i].hMem)
          if (memory[i].colour)
            GlobalFree (memory[i].hMem);
          else
            LocalFree (memory[i].hMem);
      PostQuitMessage (0);
      break;

    case WM_PAINT:
      BeginPaint (hWnd, (LPPAINTSTRUCT)&ps);
      for (i=0, lineNum = 2; i<16; i++) {
        if (memory[i].hMem) {
          itoa (i+1, szNumber, 10);
          index = StrCopy (szString, 0, szNumber, 2, TRUE);
          index = StrCopy (szString, index, "  ", 2, FALSE);
          itoa (memory[i].hMem, szNumber, 16);
          index = StrCopy (szString, index, szNumber, 4, TRUE);
          index = StrCopy(szString, index, (memory[i].colour)
                        ? ("  GLOBAL  ") : ("  LOCAL   "), 10, FALSE);
          TextOut (ps.hdc, 8, lineNum += 12, (LPSTR)szString, index);
          }
        }
      EndPaint (hWnd, (LPPAINTSTRUCT)&ps);
      break;

    case WM_INITMENU:
      hMenu = GetMenu (hWnd);
      CheckMenuItem (hMenu, item + IDITEM01, MF_CHECKED);
      if (memory[item].hMem)
        ChangeMenu (hMenu, IDALLOCATE, (LPSTR)"ReAllocate",
                IDALLOCATE, MF_CHANGE | MF_BYCOMMAND);
      else                                    /* here if item free */
        ChangeMenu (hMenu, IDALLOCATE, (LPSTR)"Allocate",
                IDALLOCATE, MF_CHANGE | MF_BYCOMMAND);
      break;

    case WM_COMMAND:
      MainWndWMCommand (hWnd, msg, wParam, lParam);
      break;
    default:
      return DefWindowProc (hWnd, msg, wParam, lParam);
      break;
    }
  return (0L);
  }

MainWndWMCommand (hWnd, msg, wParam, lParam)
  HWND          hWnd;
  unsigned      msg;
  WORD          wParam;
  LONG          lParam;
  {
  if ((wParam >= IDITEM01)
   && (wParam <= IDITEM16)) {
    hMenu = GetMenu (hWnd);
    CheckMenuItem (hMenu, item + IDITEM01, MF_UNCHECKED);
    CheckMenuItem (hMenu, wParam, MF_CHECKED);
    item = wParam - IDITEM01;
    itoa (item+1, szStr, 10);
    SetWindowText (hWnd, (LPSTR)szStr);
    }
  else {
    switch (wParam) {
      case IDALLOCATE:
        if (DialogBox(hInst, MAKEINTRESOURCE(MEMDLGBOX),
                                            hWnd, lpAllocProc))
          InvalidateRect (hWnd, (LPRECT)NULL, TRUE);
        break;


      case IDFREE:
        if (memory[item].hMem) {
          if (memory[item].colour)
            hOldMem = GlobalFree (memory[item].hMem);
          else
            hOldMem = LocalFree (memory[item].hMem);
          if (hOldMem)
            MessageBox (hWnd, (LPSTR)"Free function failed",
                            (LPSTR)NULL, MB_OK | MB_ICONHAND);
          else
            memory[item].hMem = 0;
            InvalidateRect (hWnd, (LPRECT)NULL, TRUE);
          }
        break;

      case IDDISCARD:
        if (memory[item].hMem) {
          hOldMem = FALSE;
          if (memory[item].colour) {
            if (HIBYTE(GlobalFlags(memory[item].hMem))
                                            & GMEM_DISCARDABLE)
              hOldMem = GlobalDiscard (memory[item].hMem);
            }
          else {
            if (HIBYTE(LocalFlags(memory[item].hMem))
                                            & LMEM_DISCARDABLE)
              hOldMem = LocalDiscard (memory[item].hMem);
            }
          if (hOldMem)
            MessageBox (hWnd, (LPSTR)"Discard function failed",
                             (LPSTR)NULL, MB_OK | MB_ICONHAND);
          else
            InvalidateRect (hWnd, (LPRECT)NULL, TRUE);
          }
        break;

      case IDFLAGS:
        if (memory[item].hMem) {
          index = StrCopy(szString, 0, "Lock Count: ", 12, FALSE);
          if (memory[item].colour) {
            flags = GlobalFlags(memory[item].hMem);
            itoa (flags & GMEM_LOCKCOUNT, szNumber, 10);
            index = StrCopy (szString, index, szNumber, 4, TRUE);
            if (flags & GMEM_SWAPPED)
              index = StrCopy (szString, index, "  SWAPPED",
                                                    9, FALSE);
            if (flags & GMEM_DISCARDED)
              index = StrCopy (szString, index, "  DISCARDED",
                                                    11, FALSE);
            if (flags & GMEM_DISCARDABLE)
              index = StrCopy (szString, index, "  DISCARDABLE",
                                                    13, FALSE);
           }
          else {
            flags = LocalFlags(memory[item].hMem);
            itoa (flags & LMEM_LOCKCOUNT, szNumber, 10);
            index = StrCopy (szString, index, szNumber, 4, TRUE);
            if (flags & LMEM_DISCARDED)
              index = StrCopy (szString, index, "  DISCARDED",
                                                    11, FALSE);
            if (flags & LMEM_DISCARDABLE)
              index = StrCopy (szString, index, "  DISCARDABLE",
                                                    13, FALSE);
            }
          MessageBox(hWnd, (LPSTR)szString, (LPSTR)"Flags",MB_OK);
          }
        break;

      case IDSIZE:
        if (memory[item].hMem) {
          if (memory[item].colour)
            ltoa (GlobalSize(memory[item].hMem), szNumber, 10);
          else
            itoa (LocalSize(memory[item].hMem), szNumber, 10);
          index = StrCopy (szString, 0, szNumber, 4, TRUE);
          StrCopy (szString, index, " BYTES", 6, FALSE);
          MessageBox (hWnd, (LPSTR)szString, (LPSTR)"Size",MB_OK);
          }
        break;

      case IDGLOBALCOMPACT:
        GlobalCompact ((long)-1);
        break;

      case IDLOCALCOMPACT:
        LocalCompact (-1);
        break;

      case IDLOCALFREEZE:
      case IDLOCALMELT:
        bLocalFreeze ? LocalMelt (0) : LocalFreeze (0);
        bLocalFreeze = !bLocalFreeze;
        hMenu = GetMenu (hWnd);
        EnableMenuItem (hMenu, IDLOCALFREEZE,
                    bLocalFreeze ? MF_GRAYED : MF_ENABLED);
        EnableMenuItem (hMenu, IDLOCALMELT,
                    bLocalFreeze ? MF_ENABLED : MF_GRAYED);
        break;

      case IDLOCKDATA:
      case IDUNLOCKDATA:
        bLockData ? LockData (0) : UnlockData (0);
        bLockData = !bLockData;
        hMenu = GetMenu (hWnd);
        EnableMenuItem (hMenu, IDLOCKDATA,
                    bLockData ? MF_GRAYED : MF_ENABLED);
        EnableMenuItem (hMenu, IDUNLOCKDATA,
                    bLockData ? MF_ENABLED : MF_GRAYED);
        break;

      case IDLOCK:
        if (memory[item].hMem) {
          LPSTR Addr = 0;
          if (memory[item].colour)
            Addr = GlobalLock (memory[item].hMem);
          else
            Addr = (LPSTR)LocalLock (memory[item].hMem);
          if (Addr) {
            index = StrCopy(szString, 0, "Address: ", 9, FALSE);
            if (memory[item].colour) {
              ltoa (Addr, szNumber, 16);
              StrCopy (szString, index, szNumber, 8, TRUE);
              }
            else {
              itoa (LOWORD(Addr), szNumber, 16);
              StrCopy (szString, index, szNumber, 4, TRUE);
              }
            MessageBox (hWnd, (LPSTR)szString,
                                          (LPSTR)"Lock", MB_OK);
            }
          else
            MessageBox (hWnd, (LPSTR)"Function Failed",
                                          (LPSTR)"Lock", MB_OK);
          }
        break;

      case IDUNLOCK:
        if (memory[item].hMem)
          MessageBox (hWnd,
                      ((memory[item].colour
                       ? GlobalUnlock (memory[item].hMem)
                       : LocalUnlock (memory[item].hMem))
                      ? (LPSTR)"Block still locked"
                      : (LPSTR)"Block now unlocked"),
                      (LPSTR)"Unlock",
                      MB_OK);
        break;
      }
    }
  }

/*==============================================================*/

BOOL FAR PASCAL AllocProc (hWnd, msg, wParam, lParam)
  HWND          hWnd;
  unsigned      msg;
  WORD          wParam;
  LONG          lParam;
  {
  switch (msg) {
    case WM_INITDIALOG:
      if (memory[item].hMem) {
        SetWindowText (hWnd, (LPSTR)"ReAllocate");
        SetDlgItemInt (hWnd, IDETSIZE, memory[item].size, FALSE);
        if (memory[item].colour) {
          CheckRadioButton (hWnd, IDRBGLOBAL, IDRBLOCAL, IDRBGLOBAL);
          CheckRadioButton (hWnd, IDRBFIXED, IDRBMOVEABLE,
                        (memory[item].flags & GMEM_MOVEABLE) ?
                                        IDRBMOVEABLE : IDRBFIXED);
          CheckDlgButton (hWnd, IDRBDISCARDABLE,
                (memory[item].flags & GMEM_DISCARDABLE) ? 1 : 0);
          CheckDlgButton (hWnd, IDRBNODISCARD,
              (memory[item].flags & GMEM_NODISCARD) ? 1 : 0);
          CheckDlgButton (hWnd, IDRBNOCOMPACT,
              (memory[item].flags & GMEM_NOCOMPACT) ? 1 : 0);
          }
        else {
          CheckRadioButton (hWnd, IDRBGLOBAL, IDRBLOCAL, IDRBLOCAL);
          CheckRadioButton (hWnd, IDRBFIXED, IDRBMOVEABLE,
                        (memory[item].flags & LMEM_MOVEABLE) ?
                                        IDRBMOVEABLE : IDRBFIXED);
          CheckDlgButton (hWnd, IDRBDISCARDABLE,
                (memory[item].flags & LMEM_DISCARDABLE) ? 1 : 0);
          CheckDlgButton (hWnd, IDRBNODISCARD,
              (memory[item].flags & LMEM_NODISCARD) ? 1 : 0);
          CheckDlgButton (hWnd, IDRBNOCOMPACT,
              (memory[item].flags & LMEM_NOCOMPACT) ? 1 : 0);
          }
        }
      else {
        EnableWindow (GetDlgItem (hWnd, IDOK), FALSE);
        SetWindowText (hWnd, (LPSTR)"Allocate");
        SetDlgItemText (hWnd, IDETSIZE,  (LPSTR)"");
        for (i=IDRBGLOBAL; i<=IDRBNOCOMPACT; i++)
          CheckDlgButton (hWnd, i, FALSE);
        }
      return TRUE;

    case WM_COMMAND:
      switch (wParam) {
        case IDRBGLOBAL:
        case IDRBLOCAL:
          if (!memory[item].hMem)
            CheckRadioButton (hWnd, IDRBGLOBAL, IDRBLOCAL, wParam);
          IsAllocOK (hWnd);
          break;

        case IDRBFIXED:
        case IDRBMOVEABLE:
          CheckRadioButton (hWnd, IDRBFIXED, IDRBMOVEABLE, wParam);
          if (wParam == IDRBFIXED)
            CheckDlgButton (hWnd, IDRBDISCARDABLE, FALSE);
          IsAllocOK (hWnd);
          break;


        case  IDRBDISCARDABLE:
          if (IsDlgButtonChecked (hWnd, IDRBDISCARDABLE))
            CheckDlgButton (hWnd, IDRBDISCARDABLE, FALSE);
          else
            if (IsDlgButtonChecked (hWnd, IDRBMOVEABLE))
              CheckDlgButton (hWnd, IDRBDISCARDABLE, TRUE);
          break;

        case  IDRBNOCOMPACT:
          CheckDlgButton (hWnd, IDRBNOCOMPACT,
                      !IsDlgButtonChecked (hWnd, IDRBNOCOMPACT));
          break;

        case  IDRBNODISCARD:
          CheckDlgButton (hWnd, IDRBNODISCARD,
                      !IsDlgButtonChecked (hWnd, IDRBNODISCARD));
          break;

        case IDETSIZE:
          if (HIWORD(lParam) == EN_CHANGE)
            IsAllocOK (hWnd);
          break;

        case IDOK:
          memory[item].flags = 0;
          memory[item].size =
              GetDlgItemInt (hWnd, IDETSIZE, (LPBOOL)&bValOK, FALSE);
          if (IsDlgButtonChecked (hWnd, IDRBGLOBAL)) {  /* global */
            memory[item].colour = TRUE;
            if (IsDlgButtonChecked (hWnd, IDRBMOVEABLE))
              memory[item].flags |= GMEM_MOVEABLE;
            if (IsDlgButtonChecked (hWnd, IDRBDISCARDABLE))
              memory[item].flags |= GMEM_DISCARDABLE;
            if (IsDlgButtonChecked (hWnd, IDRBNODISCARD))
              memory[item].flags |= GMEM_NODISCARD;
            if (IsDlgButtonChecked (hWnd, IDRBNOCOMPACT))
              memory[item].flags |= GMEM_NOCOMPACT;
            }
          else {                                        /* local */
            memory[item].colour = FALSE;
            if (IsDlgButtonChecked (hWnd, IDRBMOVEABLE))
              memory[item].flags |= LMEM_MOVEABLE;
            if (IsDlgButtonChecked (hWnd, IDRBDISCARDABLE))
              memory[item].flags |= LMEM_DISCARDABLE;
            if (IsDlgButtonChecked (hWnd, IDRBNODISCARD))
              memory[item].flags |= LMEM_NODISCARD;
            if (IsDlgButtonChecked (hWnd, IDRBNOCOMPACT))
              memory[item].flags |= LMEM_NOCOMPACT;
            }
          EndDialog(hWnd, memory[item].hMem =
                (memory[item].hMem
                ? (IsDlgButtonChecked (hWnd, IDRBGLOBAL)
                 ? GlobalReAlloc (memory[item].hMem, (unsigned long)
                        memory[item].size, memory[item].flags)
                 : LocalReAlloc (memory[item].hMem,
                        memory[item].size, memory[item].flags))
                : (IsDlgButtonChecked (hWnd, IDRBGLOBAL)
                 ? GlobalAlloc (memory[item].flags,
                        (unsigned long) memory[item].size)
                 : LocalAlloc (memory[item].flags,
                                        memory[item].size))));
          break;

        case IDCANCEL:
          EndDialog(hWnd, FALSE);
          break;
        }                               /* end of switch        */
      break;                            /* end of WM_COMMAND    */

    default:
      return FALSE;
    }
  return TRUE;
  }

BOOL IsAllocOK (hWnd)
  HWND  hWnd;
  {
  GetDlgItemInt (hWnd, IDETSIZE,  (LPBOOL)&bValOK, FALSE);
  if (!bValOK)
    EnableWindow (GetDlgItem (hWnd, IDOK), FALSE);
  else
    if ((IsDlgButtonChecked (hWnd, IDRBGLOBAL)
      || IsDlgButtonChecked (hWnd, IDRBLOCAL))
     && (IsDlgButtonChecked (hWnd, IDRBFIXED)
      || IsDlgButtonChecked (hWnd, IDRBMOVEABLE)))
      {
      EnableWindow (GetDlgItem (hWnd, IDOK), TRUE);
      return TRUE;
      }
  return FALSE;
  }

int StrCopy (new, newIndex, old, width, padleft)
  char  new[],  old[];
  int   width,  newIndex;
  BOOL  padleft;
  {
  int   n,      oldStrLen;

  for (oldStrLen=0; old[oldStrLen]; oldStrLen++);
  if (padleft) {
    if (width != oldStrLen)
      for (n = width - oldStrLen; n; n--, newIndex++)
        new[newIndex] = ' ';
    }
  for (n=0; new[newIndex]=old[n]; n++, newIndex++);
  return newIndex;
  }

/*==============================================================*/

int PASCAL WinMain (hInstance, hPrevInstance, lpszCmdLine, cmdShow)
  HANDLE                hInstance;
  HANDLE                hPrevInstance;
  LPSTR                 lpszCmdLine;
  int                   cmdShow;
  {
  MSG                   msg;
  HWND                  hWnd;

  if (!hPrevInstance) {
    if (!MainInit (hInstance))
      return FALSE;
    }

  hWnd = CreateWindow ((LPSTR)"Demo", (LPSTR)"1",
                WS_TILEDWINDOW, 0, 0, 0, 0, (HWND)NULL, (HMENU)NULL,
                (HANDLE)hInstance,  (LPSTR)NULL);
  hInst     = hInstance;
  lpAllocProc = MakeProcInstance ((FARPROC)AllocProc, hInstance);

  ShowWindow (hWnd, cmdShow);
  UpdateWindow (hWnd);

  while (GetMessage((LPMSG)&msg, NULL, 0, 0)) {
    TranslateMessage ((LPMSG)&msg);
    DispatchMessage ((LPMSG)&msg);
    }
  return (int)msg.wParam;
  }

BOOL MainInit (hInstance)
  HANDLE        hInstance;
  {
  PWNDCLASS pMainClass = (PWNDCLASS)LocalAlloc(LPTR,sizeof(WNDCLASS));
  pMainClass->hCursor        = LoadCursor (NULL, IDC_ARROW);
  pMainClass->hIcon          = LoadIcon (NULL, IDI_APPLICATION);
  pMainClass->lpszMenuName   = (LPSTR)"MainMenu";
  pMainClass->lpszClassName  = (LPSTR)"Demo";
  pMainClass->hbrBackground  = (HBRUSH)GetStockObject (WHITE_BRUSH);
  pMainClass->hInstance      = hInstance;
  pMainClass->style          = CS_HREDRAW | CS_VREDRAW;
  pMainClass->lpfnWndProc    = MainWndProc;
  if (!RegisterClass((LPWNDCLASS)pMainClass))
    return FALSE;
  LocalFree ((HANDLE)pMainClass);
  return TRUE;
  }
/*--------------------------------------------------------------*/
/* End of file:  MEMMGR.C                                       */
/****************************************************************/
